/** 
 *  Yudit Unicode Editor Source File
 *
 *  Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gui/SMessageBar.h"

#include "gui/Main.h"
#include "swidget/SIconFactory.h"


/**
 * a button panel is just a panel of labels.
 */
SMessageBar::SMessageBar (const SProperties props)
{
  SColor fg = SColor(props["yudit.label.foreground"]);
  SColor tfg = SColor(props["yudit.title.foreground"]);
  SString font = props["yudit.default.font"];
  SString fontSizeString=props["yudit.default.fontsize"];

  double fontSize=0;
  fontSizeString.append ((char)0);
  sscanf (fontSizeString.array(), "%lf", &fontSize);
  if (fontSize < 2.0) fontSize = 2.0;

  /* create the labels and set the preferred size */
  SLabel *l;

  l = new SLabel (translate ("Glyph Info:"));
  l->setFont(font, fontSize);
  l->setForeground (tfg);
  labels[SS_GLYPH_TITLE] = l;

  l = new SLabel (translate (""));
  l->setFont(font, fontSize);
  l->setForeground (fg);
  labels[SS_GLYPH_INFO] = l;

  l = new SLabel (translate ("Font Size:"));
  l->setFont(font, fontSize);
  l->setForeground (tfg);
  labels[SS_FONT_SIZE_TITLE] = l;

  l = new SLabel (translate (""));
  l->setFont(font, fontSize);
  l->setForeground (fg);
  labels[SS_FONT_SIZE] = l;

  l = new SLabel (translate ("Line:"));
  l->setFont(font, fontSize);
  l->setForeground (tfg);
  labels[SS_ROW_TITLE] = l;

  l =  new SLabel ("");
  l->setFont(font, fontSize);
  l->setForeground (fg);
  labels[SS_ROW] = l;

  l = new SLabel (translate ("Column:"));
  l->setFont(font, fontSize);
  l->setForeground (tfg);
  labels[SS_COLUMN_TITLE] = l;

  l =  new SLabel ("");
  l->setFont(font, fontSize);
  l->setForeground (fg);
  labels[SS_COLUMN] = l;

  unsigned int p=0;
  unsigned int h=0;
  for (unsigned int i=0; i<(unsigned int) SS_MAX; i++)
  {
    unsigned int ww = 0;
    switch (i)
    {
    case SS_FONT_SIZE:
      ww = 50;
      break;
    case SS_ROW:
      ww = 50;
      break;
    case SS_COLUMN:
      ww = 50;
      break;
    case SS_GLYPH_INFO:
      ww = 200;
      break;
    default:
      ww = labels[i]->getPreferredSize().width;
    }
    unsigned int hh = labels[i]->getPreferredSize().height;
    labels[i]->resize(SDimension (ww, hh));
    labels[i]->move(SLocation ((int)p, 0));
    add (labels[i]);
    p+= ww +2;
    if (hh > h) h = hh;
  }
  preferredSize = SDimension (p, h);
}

SMessageBar::~SMessageBar ()
{
}


void
SMessageBar::setRow (unsigned int row)
{
  char a[64];
  sprintf (a, "%u", row);
  SLabel* l = labels[SS_ROW];
  l->setText (a);
}

void
SMessageBar::setColumn (unsigned int column)
{
  char a[64];
  sprintf (a, "%u", column);
  SLabel* l = labels[SS_COLUMN];
  l->setText (a);
}

void
SMessageBar::setFontSize (double d)
{
  char a[64];
  sprintf (a, "%g", d);
  fontSize = d;
  SLabel* l = labels[SS_FONT_SIZE];
  l->setText (a);
}

void
SMessageBar::setGlyphInfo (const SGlyph* g)
{
  SLabel* l = labels[SS_GLYPH_INFO];
  if (g==0)
  {
    l->setText("");
    return;
  }
  SV_UCS4 c;
  for (unsigned int i=0; i<g->size(); i++)
  {
    c.append ((*g)[i]);
  }
  SString decomposition = unicodeValueOf (c);
  if (g->composition == 0)
  {
    l->setText (decomposition);
    return;
  }
  c.clear();
  c.append (g->composition);
  SString composition = unicodeValueOf (c);
  SString text;
  if (g->useComposition)
  {
     text.append (composition);
  }
  else
  {
     text.append (decomposition);
  }
  text.append (" [");
  if (g->useComposition)
  {
     text.append (decomposition);
  }
  else
  {
     text.append (composition);
  }
  text.append ("]");
  l->setText(text);
}
