/** 
 *  Yudit Unicode Editor Source File
 *
 *  Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#ifndef SBVector_h
#define SBVector_h

/**
 * @author: Gaspar Sinai <gsinai@yudit.org>
 * @version: 2000-04-23
 */
#include "SObject.h"
#include "SShared.h"
#include "SExcept.h"

class SBVector : public SObject
{
public:
  inline SBVector(void);
  virtual ~SBVector ();
  SBVector (const SBVector& v);
  SBVector& operator=(const SBVector& v);
  virtual SObject* clone() const 
  { SBVector* n = new SBVector (*this); CHECK_NEW(n); return n;}
  
  /**
   * insert, replace, replaceAll, remove, clear require that you properly 
   * derefer objects.
   */
  void  insert (unsigned int index, const char* e, unsigned int len);
  void  replace (unsigned int index, const char* e, unsigned int len);
  void  remove (unsigned int index, unsigned int len);
  void  clear ();

  int   replace (const char* e, unsigned int len, 
             const char* with, unsigned int withLen, unsigned int from=0, unsigned int align = 1);
  int   replaceAll (const char* e, unsigned int len, 
             const char* with, unsigned int  withLen, unsigned int from=0, unsigned int align = 1);

  inline char* peek (unsigned int index) const;
  int   find (const char* e, unsigned int len, unsigned int from, unsigned int align=1) const;

  inline unsigned int  size () const;
  static int    debug (int level);
  inline bool isNull () const;
  bool equals (const SBVector& v) const;
  inline const char* array() const;
protected:
  inline unsigned int count () const;
  inline void  derefer ();
  void  refer(const SBVector& v);
  void  ensure (unsigned int more);
private:
  SShared*  buffer;
  inline void  cleanup ();
};

/**
 * This is the base for all object vectors
 */
class SOVector : public SBVector
{
public:
  SOVector (void);
  SOVector (const SOVector& v);
  virtual ~SOVector ();
  virtual SObject* clone() const;
  SOVector& operator=(const SOVector& v);

  void replace (unsigned int index, const SObject& v);
  void insert (unsigned int index, const SObject& v);
  void remove (unsigned int index);

  unsigned int size () const;

  void clear ();
  void refer (const SOVector& v);
  const SObject* peek (unsigned int index) const;

protected:
  void derefer();

private:
  void replaceQuetly (unsigned int index, const SObject* r);
  void cleanup();
};


//PUBLIC/INLINE
/**
 * This is the vector class. I dont prefer using STL.
 */
SBVector::SBVector(void)
{
    buffer = new SShared();
}


//PUBLIC/INLINE
/**
 * Return the size of the char array
 */
unsigned int
SBVector::size() const
{
  return buffer->vectorSize;
}


//PUBLIC/INLINE
bool
SBVector::isNull () const
{
  return buffer->arraySize==0;
}

//PUBLIC/INLINE
const char*
SBVector::array() const
{
  return buffer->array;
}

//PUBLIC/INLINE
/**
 * Return the element at index.
 * @param index is the reference index of len blocks
 */
char*
SBVector::peek (unsigned int index) const
{
  if (index > buffer->vectorSize) return 0;
  return &buffer->array[index];
}


//PROTECTED/INLINE
unsigned int
SBVector::count() const
{
  return buffer->count;
}

#endif /* SBVector_h */
