/** 
 *  Yudit Unicode Editor Source File
 *
 *  Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#include "SShared.h"
#include "SExcept.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

static int rem=0;
static int debug_level=0;

/**
 * @author: Gaspar Sinai <gsinai@yudit.org>
 * @version: 2000-04-23
 */

/**
 * Create a buffer that will be referenced by all vectors.
 */
SShared::SShared()
{
  count=1; array=0; arraySize=0; vectorSize=0;
  ++rem;
}

/**
 * Create a buffer that will be referenced by all vectors.
 * @param size is the minimum size on chars.
 */
SShared::SShared (unsigned int size)
{
  count=1; vectorSize=0;
  array = new char[size];
  CHECK_NEW(array);
  arraySize = size;
  ++rem;
}

/**
 * Create a buffer that will be referenced by all vectors.
 * @param size is the minimum size on chars.
 * @param buffer - this will be copyes over one by one.
 */
SShared::SShared (const char* buffer, unsigned int size)
{
  count=1; array = 0;  arraySize=0; vectorSize=0;
  if (size==0)return;
  
  array = new char[size];
  CHECK_NEW(array);
  arraySize = size;
  //ensure (arraySize);
  memcpy (array, buffer, size);
  ++rem;
}

/**
 * Create a new Object by copying an old one
 * @param a SShared object to copy
 */
SShared::SShared (const SShared& orig)
{
  count=1; array=0; arraySize=0; vectorSize=0;
  ++rem;
  /* Copy */
  if (orig.arraySize!=0)
  {
  	ensure (orig.arraySize);
  	memcpy (array, orig.array, orig.arraySize);
  }
  vectorSize = orig.vectorSize;
}

/**
 * delete a buffer. The count should be one here!
 */
SShared::~SShared()
{
  rem--;
  if (arraySize != 0) {
    delete array;
  }
  if (debug_level && rem==0)
  {
     fprintf (stderr, "Debug OK: SShared::~SShared no memory leak.\n");
  }
}

/**
 * Sets debug printout levels
 */
int
SShared::debug(int level)
{
  int prev = debug_level;
  debug_level = level;
  return prev;
}

/**
 * Ensure that we have enough capacity.
 * Before this, the reference count should be one!
 * int more - the elements in bytes that we need.
 */
void
SShared::ensure(unsigned int more)
{
  if (arraySize > vectorSize + more) return;
  if (count != 1)
  {	
     fprintf (stderr, "Internal error in SBVector::ensure\n");
     exit(1);
  }

  char* oldArray=array;
  unsigned int newSize = (vectorSize == 0) ? more : (more + arraySize) + arraySize/2; 
  if (newSize==0) newSize=1;
  array = new char[newSize];
  CHECK_NEW(array);

  // We copy the whole thing. DOnt care if arraySize is different.
  /* hashtable does not use vectorSize */
  if (arraySize)
  {
    memcpy (array, oldArray, arraySize);
    delete oldArray;
  }
  // Hashtable has a fixed size. If resized, ever we need this line 
  //memset (&array[arraySize], 0, newSize - arraySize);
  arraySize = newSize;
}
