/** 
 *  Yudit Unicode Editor Source File
 *
 *  Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "swidget/SDialog.h"
#include "swidget/SIconFactory.h"
#include "stoolkit/SUtil.h"

SDialog::SDialog (SPanel* panel)
{
  dialogPanel = panel;
  setFrameListener (this);
  isCancel = true;
  type = SS_NONE;
  yesButton = new SButton("");
  noButton = new SButton ("");
  yesButton->setButtonListener (this);
  yesButton->setIcon (SIconFactory::getIcon("Yes"));
  noButton->setButtonListener (this);
  noButton->setText (translate ("No"));
  noButton->setIcon (SIconFactory::getIcon("Cancel"));
  add (dialogPanel);
  add (yesButton);
  add (noButton);
}

SDialog::~SDialog ()
{
}

bool
SDialog::getInput (SType  messageType)
{
  type = messageType;
  switch (type)
  {
  case SS_ERR: 
    noButton->setText (translate ("No"));
    yesButton->setText (translate ("OK"));
    break;
  case SS_WARN:
    noButton->setText (translate ("No"));
    yesButton->setText (translate ("OK"));
    break;
  case SS_INFO:
    noButton->setText (translate ("No"));
    yesButton->setText (translate ("OK"));
    break;
  case SS_QUESTION:
    noButton->setText (translate ("No"));
    yesButton->setText (translate ("Yes"));
    break;
  case SS_OK_CANCEL:
    noButton->setText (translate ("Cancel"));
    yesButton->setText (translate ("OK"));
    break;
  default:
    break;
  }

  recalc (); resize (preferredSize); center ();
  isCancel = true;
  show ();
  wait();
  hide();
  return !isCancel;
}

/**
 * recalculate geometry
 */
void
SDialog::recalc ()
{
  SDimension dl = dialogPanel->getPreferredSize();
  SDimension db = yesButton->getPreferredSize();
  SDimension dn = noButton->getPreferredSize();

  unsigned int wd = db.width;
  if (type == SS_QUESTION || type == SS_OK_CANCEL)
  {
    if (dn.width > db.width) db.width = dn.width;
    if (dn.width < db.width) dn.width = db.width;
    wd  = db.width + dn.width;
  }
  if (wd < dl.width) wd = dl.width;
  preferredSize = SDimension (wd + 15, db.height + dl.height + 20);

  setLayout (SLayout (SLocation (0,0), 
      SLocation (preferredSize.width,preferredSize.height)));
  dialogPanel->setLayout (
    SLayout (
       SLocation (5,5),
       SLocation (preferredSize.width-5, 
             preferredSize.height-10-(int)db.height),
       SLocation (0,0),
       SLocation (100,100)
    )
  );
  if (type != SS_QUESTION && type != SS_OK_CANCEL)
  {
    noButton->setLayout(
      SLayout (
        SLocation (-200, -200),
        SLocation (-100, -100)
      )
    );
    noButton->hide();
    yesButton->setLayout (
      SLayout (
        SLocation (5 + ((int)preferredSize.width-10)/2 - (int)db.width/2, 
             (int)preferredSize.height - 5 - (int)db.height),
        SLocation (5 + ((int)preferredSize.width-10)/2 + (int)db.width/2, 
             (int)preferredSize.height-5),
        SLocation (50,100),
        SLocation (50,100)
      )
    );
  }
  else
  {
    noButton->show();
    yesButton->setLayout (
      SLayout (
        SLocation (5, (int)preferredSize.height - 5 - (int)db.height),
        SLocation (5 + (int) db.width, (int)preferredSize.height-5),
        SLocation (50,100),
        SLocation (50,100)
      )
    );
    noButton->setLayout (
      SLayout (
        SLocation (5 + ((int)preferredSize.width-10) - (int)dn.width, 
             (int)preferredSize.height - 5 - (int)dn.height),
        SLocation (5 + ((int)preferredSize.width-10), 
             (int)preferredSize.height-5),
        SLocation (50,100),
        SLocation (50,100)
      )
    );
  }
  setMinimumSize(preferredSize);
}

void
SDialog::buttonPressed (void* source, const SAccelerator* acc)
{
  isCancel = false;
  if (source == noButton)
  {
     isCancel =true;
  }
  if (source == yesButton)
  {
     isCancel =false;
  }
  hide ();
}
/**
 * from frameListener
 */
bool
SDialog::close (SPanel* comp)
{
  isCancel =true;
  hide();
  return false;
}
void
SDialog::setBackground (const SColor& bg)
{
  SFrame::setBackground (bg);
}

void 
SDialog::setTitleForeground (const SColor& fg)
{
  yesButton->setForeground (fg);
  noButton->setForeground (fg);
}

void
SDialog::setFont (const SString& font, double fontSize)
{
  yesButton->setFont (font, fontSize);
  noButton->setFont (font, fontSize);
}
void
SDialog::setFontSize (double fontSize)
{
  yesButton->setFontSize (fontSize);
  noButton->setFontSize (fontSize);
}

/**
 * Hide the dialog if this is not a question dialog.
 */
void
SDialog::keyPressed (SWindow * w, SKey key, const SString& s,
          bool ctrl, bool shift, bool meta)
{
  if (type == SS_QUESTION || type == SS_OK_CANCEL) return;
  if (key == Key_Enter || key == Key_Return) 
  {
     hide ();
  }
}
