/** 
 *  Yudit Unicode Editor Source File
 *
 *  Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "swidget/SListBox.h"
SListBox::SListBox (const SString& title)
{
 topLabel = new SLabel (title);
 topLabel->setAlignment (SD_Center);
 textList = new STextList();
 slider = new SSlider();
 SPanel::setLayout (SLayout (SLocation (0,0), SLocation (300,200)));
 recalc ();
 add (topLabel);
 add (textList);
 add (slider);
}

void 
SListBox::resize(const SDimension& d)
{
  SPanel::resize (d);
  border.resize(d);
}

void
SListBox::redraw (SWindow *canvas, int x, int y, 
  unsigned int width, unsigned int height)
{
  border.redraw(canvas, x, y, width, height);
}

SListBox::~SListBox ()
{
}

void
SListBox::setText (const SStringVector& v)
{
  textList->setText (v);
}
void
SListBox::setListListener (SListListener* l)
{
  textList->setListListener (l);
}
void
SListBox::setLayout (const SLayout& _layout)
{
 SPanel::setLayout (_layout);
 recalc ();
}

/**
 * recalculate the layout
 */
void
SListBox::recalc()
{
  SDimension d = layout.getDimension ();
  unsigned int lh = topLabel->getPreferredSize().height;
  SDimension bd = border.getBorderSize();

  topLabel->setLayout (
    SLayout (
      SLocation (bd.width,bd.height),
      SLocation (d.width-2*(int)bd.width, lh+(int)bd.height),
      SLocation (0, 0),
      SLocation (100, 0)
    )
  );

  unsigned int th = (d.height > lh ) ? d.height - lh : 1;
  unsigned int tw = (d.width > 20) ? d.width - 20 : 1;

  textList->setLayout (
    SLayout (
      SLocation (0+bd.width, lh+bd.height),
      SLocation (tw-2*(int)bd.width, lh+th-2*(int)bd.height),
      SLocation (0, 0),
      SLocation (100, 100)
    )
  );  
  slider->setLayout (
    SLayout (
      SLocation (tw-2*(int)bd.width, lh+(int)bd.height),
      SLocation (d.width-2*(int)bd.width, lh+th-2*(int)bd.height),
      SLocation (100, 0),
      SLocation (100, 100)
    )
  );
  preferredSize = SDimension (topLabel->getPreferredSize().width + 4,
        topLabel->getPreferredSize().height*2 + 4);
}

void
SListBox::setBackground (const SColor& bg)
{
  SPanel::setBackground (bg);
  border.setBackground (bg);
}

bool
SListBox::selectText (const SString& s)
{
  return textList->selectText (s);
}

bool
SListBox::selectItem (int item)
{
  return textList->selectItem (item);
}

void
SListBox::setLabelForeground (const SColor& fg)
{
  topLabel->setForeground (fg);
}

void
SListBox::setSliderBackground (const SColor& bg)
{
  slider->setSliderBackground (bg);
}

void
SListBox::setFont (const SString& font, double fontSize)
{
  topLabel->setFont (font, fontSize);
  textList->setFont (font, fontSize);
  recalc();
}
void
SListBox::setFontSize (double fontSize)
{
  topLabel->setFontSize (fontSize);
  textList->setFontSize (fontSize);
  recalc();
}
