/** 
 *  Yudit Unicode Editor Source File
 *
 *  Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#ifndef SFontImpl_h
#define SFontImpl_h

#include "swindow/SCanvas.h"
#include "swindow/SPen.h"

#include "stoolkit/SMatrix.h"
#include "stoolkit/SStringVector.h"
#include "stoolkit/SObject.h"


/**
 * @author: Gaspar Sinai <gsinai@yudit.org>
 * @version: 2000-04-23
 * This is the abstract widget toolkit font package
 */
class SFontImpl : public SObject
{
public:
  SFontImpl (const SString& name, const SString& xlfd); 
  SFontImpl (const SFontImpl& ff);
  SFontImpl operator= (const SFontImpl& ff);
  virtual ~SFontImpl (); 

  static void setPath (const SStringVector& path);
  static void guessPath(const SStringVector& files, const SString& property);
  static void guessPath();

  virtual SObject*  clone() const;

  void   scale (double x, double y);

  unsigned int draw (SCanvas* canvas, const SPen& pen, 
       const SS_Matrix2D& matrix, const SS_UCS4* uch, unsigned int len);

  double width (const SS_UCS4* uch, unsigned int len, unsigned int* used);

  double width () const;

  double ascent () const;

  double descent () const;

  double gap () const;

private:
  void createSaneXLFD ();
  SString name;
  SString xlfd;
  SS_Matrix2D matrix;
  void* delegate;
};

#endif /* SFontImpl_h */
