/** 
 *  Yudit Unicode Editor Source File
 *
 *  Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#include "swindow/SFontTTF.h"

#include "stoolkit/SStringVector.h"
#include "stoolkit/SExcept.h"
#include "stoolkit/SUniMap.h"
/*
 * ntohl 
 */
#ifndef USE_WINAPI
#include <netinet/in.h>
#else
#include <winsock.h>
#endif

#include <stdio.h>
#include <ctype.h>

/**
 * This is a hash width key platform '~' encoding 
 * and value SUnimap.
 */
static SProperties encodingHashtable;
static SStringVector encodingVector ("ucs2 plat=3 enc=1,ucs2 ptat=1,ucs2 plat=2 enc=1");

static const SString SS_TB_NAME("name");
static const SString SS_TB_HEAD("head");
static const SString SS_TB_HHEA("hhea");
static const SString SS_TB_POST("post");
static const SString SS_TB_GLYF("glyf");
static const SString SS_TB_CMAP("cmap");
static const SString SS_TB_KERN("kern");
static const SString SS_TB_MAXP("maxp");
static const SString SS_TB_HTMX("hmtx");
static const SString SS_TB_LOCA("loca");

static const long SS_TN_NOTICE=0;
static const long SS_TN_FAMILY=1;
static const long SS_TN_WEIGHT=2;
static const long SS_TN_X3=3;
static const long SS_TN_FULLNAME=4;
static const long SS_TN_VERSION=5;
static const long SS_TN_FONTNAME=6;
static const long SS_TN_X7=7;
static const long SS_TN_MAX=8;


/*
 * The following definitions were derived from
 * Andrew Weeks's excellent font converter program.
 * Many thanks!
 */
#define BYTE unsigned char
#define CHAR signed char
#define USHORT unsigned short
#define SHORT signed short
#define ULONG unsigned int
#define LONG signed int
#define FWORD SHORT
#define UFWORD USHORT

#define ONOROFF 0x01
#define XSHORT  0x02
#define YSHORT  0x04
#define REPEAT  0x08
#define XSAME   0x10
#define YSAME   0x20
#define ARG_1_AND_2_ARE_WORDS           0x0001
#define ARGS_ARE_XY_VALUES                      0x0002
#define XY_BOUND_TO_GRID                        0x0004
#define WE_HAVE_A_SCALE                         0x0008
#define MORE_COMPONENTS                         0x0020
#define WE_HAVE_AN_X_AND_Y_SCALE        0x0040
#define WE_HAVE_A_TWO_BY_TWO            0x0080
#define WE_HAVE_INSTRUCTIONS            0x0100
#define USE_MY_METRICS                          0x0200

typedef struct short_2 {
        SHORT   upper;
        USHORT  lower;
} SSFIX ;

typedef struct longhormetric {
        UFWORD  advanceWidth;
        FWORD   lsb;
} LONGHORMETRIC;

typedef struct ttf_hhea {
        BYTE    version[4];
        SHORT   ascender, descender, lineGap;
        USHORT  advnaceWidthMax;
        SHORT   minLSB, minRSB, xMaxExtent;
        SHORT   caretSlopeRise, caretSlopeRun;
        SHORT   reserved[5];
        SHORT   metricDataFormat;
        USHORT  numberOfHMetrics;
} TTF_HHEA;

typedef struct ttf_dir_entry {
        char    tag[4];
        ULONG   checksum;
        ULONG   offset;
        ULONG   length;
} TTF_DIR_ENTRY ;

typedef struct ttf_directory {
        ULONG                   sfntVersion;
        USHORT                  numTables;
        USHORT                  searchRange;
        USHORT                  entrySelector;
        USHORT                  rangeShift;
        TTF_DIR_ENTRY   list;
} TTF_DIRECTORY ;

typedef struct ttf_name_rec {
        USHORT  platformID;
        USHORT  encodingID;
        USHORT  languageID;
        USHORT  nameID;
        USHORT  stringLength;
        USHORT  stringOffset;
} TTF_NAME_REC;

typedef struct ttf_name {
        USHORT                  format;
        USHORT                  numberOfNameRecords;
        USHORT                  offset;
        TTF_NAME_REC    nameRecords;
} TTF_NAME ;

typedef struct ttf_head {
        ULONG   version;
        ULONG   fontRevision;
        ULONG   checksumAdjust;
        ULONG   magicNo;
        USHORT  flags;
        USHORT  unitsPerEm;
        BYTE    created[8];
        BYTE    modified[8];
        FWORD   xMin, yMin, xMax, yMax;
        USHORT  macStyle, lowestRecPPEM;
        SHORT   fontDirection, indexToLocFormat, glyphDataFormat;
} TTF_HEAD ;
typedef struct ttf_kern {
        USHORT  version, nTables;
} TTF_KERN ;

typedef struct ttf_kern_sub {
        USHORT version, length, coverage;
        USHORT nPairs, searchRange, entrySelector, rangeShift;
} TTF_KERN_SUB;

typedef struct ttf_kern_entry {
        USHORT  left, right;
        FWORD   value;
} TTF_KERN_ENTRY;

typedef struct ttf_cmap_fmt0 {
        USHORT  format;
        USHORT  length;
        USHORT  version;
        BYTE    glyphIdArray[256];
} TTF_CMAP_FMT0;

typedef struct ttf_cmap_fmt4 {
        USHORT  format;
        USHORT  length;
        USHORT  version;
        USHORT  segCountX2;
        USHORT  searchRange;
        USHORT  entrySelector;
        USHORT  rangeShift;
} TTF_CMAP_FMT4;

typedef struct ttf_cmap_entry {
        USHORT  platformID;
        USHORT  encodingID;
        ULONG   offset;
} TTF_CMAP_ENTRY;

typedef struct ttf_cmap {
        USHORT                  version;
        USHORT                  numberOfEncodingTables;
        TTF_CMAP_ENTRY  encodingTable[1];
} TTF_CMAP ;

typedef struct ttf_glyf {
        SHORT   numberOfContours;
        FWORD   xMin, yMin, xMax, yMax;
} TTF_GLYF ;

typedef struct ttf_maxp {
        ULONG   version;
        USHORT  numGlyphs, maxPoints, maxContours;
        USHORT  maxCompositePoints, maxCompositeContours;
        USHORT  maxZones, maxTwilightPoints, maxStorage;
        USHORT  maxFunctionDefs, maxInstructionsDefs;
        USHORT  maxSizeOfInstructions, maxComponentElements;
        USHORT  maxComponentDepth;
} TTF_MAXP ;

typedef struct ttf_post_head {
        ULONG   formatType;
        SSFIX   italicAngle;
        FWORD   underlinePosition;
        FWORD   underlineThickness;
        ULONG   isFixedPitch;
        ULONG   minMemType42;
        ULONG   maxMemType42;
        ULONG   minMemType1;
        ULONG   maxMemType1;
        USHORT  numGlyphs;
        USHORT  glyphNameIndex;
} TTF_POST_HEAD ;

static double f2dot14 (short x);
static void moveto (SCanvas* canvas, const SS_Matrix2D &m,
  SHORT _x, SHORT _y);
static void lineto (SCanvas* canvas, const SS_Matrix2D &m,
  SHORT _x, SHORT _y);
static void cureveto (SCanvas* canvas, const SS_Matrix2D &m,
  SHORT _x0, SHORT _y0,
  SHORT _x1, SHORT _y1,
  SHORT _x2, SHORT _y2);

/**
 * @author: Gaspar Sinai <gsinai@yudit.org>
 * @version: 2000-04-23
 * Many parts of this file are originally written by Andrew Weeks.
 */


/**
 * Initialize a TTF
 * @param name is the font file
 */
SFontTTF::SFontTTF (const SFile& _file) : file (_file)
{
  ok = true;
  fastMap =0;
}

SFontTTF::~SFontTTF ()
{
  if (fastMap) delete fastMap;
}

bool
SFontTTF::isOK()
{
  if (!ok) return false;

  if (name.size()==0)
  {
    name = file.getName();
//fprintf (stderr, "Setting name to %*.*s\n", SSARGS(name));
    if (file.size() < 0)
    {
      ok = false;
    }
    else
    {
      image = file.getFileImage();
      ok = image.size()>0 && init();
    }
  }
  if (ok && fastMap ==0)
  {
    fastMap = new unsigned char[0x10000/4];
    CHECK_NEW(fastMap);
    memset (fastMap, 0, 0x10000/4);
  }
  return ok;
}

/**
 * initialize all numbers.
 * return false if something is wrong with this font.
 */
bool
SFontTTF::init ()
{
  TTF_DIRECTORY* directory = (TTF_DIRECTORY *) image.array();
  if (ntohl (directory->sfntVersion) != 0x00010000)
  {
    fprintf (stderr, "SFontTTF: BAD TTF file [%*.*s].\n", SSARGS(name));
    return false;
  }
  TTF_DIR_ENTRY* dir_entry = &(directory->list);
  
  char tag[5];
  unsigned int i;
  for (i=0; i < (unsigned short)ntohs(directory->numTables); i++)
  {
    for (unsigned int j=0; j<4; j++)
    {
      tag[j] = dir_entry->tag[j];
    }
    tag[4] = 0;
    tables.put (tag, image.array() + ntohl (dir_entry->offset));
    if (memcmp(tag, "EBDT", 4)==0 || memcmp(tag, "EBLC", 4)==0
        || memcmp(tag, "EBSC", 4)==0)
    {
      //fprintf (stderr, "SFontTTF info: TTF file [%*.*s] contains bitmaps.\n",
       //  SSARGS(name));
    }
    dir_entry++;
  }
  if (!processName())
  {
     return false;
  }
  if (!checkTables())
  {
     return false;
  }
  broken = false;
  if (encodingVector.size() != 0)
  {
    for (unsigned int i=0; i<encodingVector.size(); i++)
    {
      SStringVector pe;
      pe.split (encodingVector[i], " ", true);
      if (pe.size() != 2) continue;
      encodingHashtable.put (pe[1], pe[0]);
//fprintf (stderr, "SFontTTF: init %*.*s -> %*.*s.\n", SSARGS(pe[1]), SSARGS(pe[0]));
    }
    encodingVector.clear();
  }

  TTF_CMAP* cmap_table = (TTF_CMAP*) tables[SS_TB_CMAP];
  int num_tables = ntohs(cmap_table->numberOfEncodingTables);
  SStringVector maps; 

  lengthIsOne = true;
  for (i=0; i < (unsigned int) num_tables; i++)
  {
    TTF_CMAP_ENTRY* table_entry = &(cmap_table->encodingTable[i]);
    int offset = ntohl(table_entry->offset);
    TTF_CMAP_FMT4* encoding4 = (TTF_CMAP_FMT4 *) ((BYTE *)cmap_table + offset);
    int format = ntohs(encoding4->format);
    int platform = ntohs(table_entry->platformID);
    int encoding_id = ntohs(table_entry->encodingID);

    /* I can understand only this format... */
    SUniMap umap;
    if (format != 4)
    {
//fprintf (stderr, "for %d using default.\n", i);
      umaps.append (umap);
      continue;
    }
    char pe[64];
    sprintf (pe, "plat=%d enc=%d", platform, encoding_id);
    maps.append (pe);
    SString map;
    if (encodingHashtable.get (pe) != 0)
    {
      map = encodingHashtable[pe];
    }
    else
    {
      sprintf (pe, "plat=%d", platform);
      if (encodingHashtable.get (pe) ==0)
      {
         umaps.append (umap);
         continue;
      }
      map = encodingHashtable[pe];
    }
    if (map != SString("ucs2"))
    {
      umap = SUniMap(map);
    }
//fprintf (stderr, "for %d using %*.*s\n", i, SSARGS(map));
    if (!umap.isOK())
    {
      fprintf (stderr, "SFontTTF: umap '%*.*s' not found for '%*.*s'.\n",
        SSARGS(map), SSARGS(name));
    }
    umaps.append (umap);
    /* FIXME: check lengthIsOne */
  }
  //fprintf (stderr, "init: num_tables=%d mapsize=%u\n", num_tables, umaps.size());
   
  return true;
}


/**
 * Process the name table
 */
bool
SFontTTF::processName ()
{
  TTF_NAME*  name_table = (TTF_NAME*) tables[SS_TB_NAME];
  if (name_table==0)
  {
    getName (SS_TN_FONTNAME, name.array(), name.size());
    fprintf (stderr, "SFontTTF: No name fields in %*.*s\n", SSARGS(name));
    fprintf (stderr, "SFontTTF: records:[");
    for (unsigned int i=0; i<tables.size(); i++)
    {
      for (unsigned int j=0; j<tables.size(i); j++)
      {
        SString key = tables.key(i, j);
        fprintf (stderr, " %*.*s", SSARGS(key));
        
      }
    }
    fprintf (stderr, " ]\n");
    return false;
  }

  TTF_NAME_REC* name_record = &(name_table->nameRecords);
  char* string_area = (char *)name_table + ntohs(name_table->offset);
    
  int found=0;
  int i;
  for (i=0; i < ntohs (name_table->numberOfNameRecords); i++) 
  {
    short platform = ntohs(name_record->platformID);
    if (platform == 3)
    {
      found = 1;
      short lang = ntohs(name_record->languageID) & 0xff;
      short len = ntohs(name_record->stringLength);
      short strOffset = ntohs(name_record->stringOffset);
      if (lang == 0 || lang == 9)
      {
        long nameId = ntohs(name_record->nameID);
        if (nameId < SS_TN_MAX)
        {
          getName (nameId, &string_area[strOffset], len);
        }
      }
    }
    name_record++;
  }

  name_record = &(name_table->nameRecords);
  if (!found) for (i=0; i < ntohs(name_table->numberOfNameRecords); i++) 
  {
   short platform = ntohs(name_record->platformID); if (platform ==1)
   {
      found = 1;
      short lang = ntohs(name_record->languageID) & 0xff;
      short len = ntohs(name_record->stringLength);
      short strOffset = ntohs(name_record->stringOffset);
      if (lang == 0 || lang == 9)
      {
        long nameId = ntohs(name_record->nameID);
         if (nameId < SS_TN_MAX)
         {
           getName (nameId, &string_area[strOffset], len);
         }
      }
    }
    name_record++;
  }
  if (!found)
  {
    fprintf (stderr, "SFontTTF: BAD Name fileds in %*.*s\n", SSARGS(name));
    return false;
  }

  if (names.get (SS_TN_FONTNAME) == 0 || names[SS_TN_FONTNAME].size() == 0)
  {
    getName (SS_TN_FONTNAME, names[SS_TN_FULLNAME].array(),
         names[SS_TN_FULLNAME].size());
  }
  return true;
}

/**
 * put the string from str into names. 
 * @param id is SS_TN_ something.
 * @param str is the input string
 * @param len is the size of the stirng
 */
void
SFontTTF::getName (long id, const char* str, int len)
{
  SString s;
  for (int i=0; i<len; i++)
  {
    if (str[i] == 0) continue;
    if (id==SS_TN_FONTNAME)
    {
      if (isalnum(str[i]))
      {
        s.append (str[i]);
      }
      else
      {
        s.append ((char)'_');
      }
    }
    /* This is to make postscript files clean */
    else switch (str[i])
    {
    case '(':
      s.append ((char)'[');
      break;
    case ')':
      s.append ((char)']');
      break;
    default:
      s.append (str[i]);
    }
  }
  names.put (id, s);
}

/**
 * Do a sanity check.all tables
 */
bool
SFontTTF::checkTables ()
{

  if (tables[SS_TB_HEAD] == 0)
  {
    fprintf (stderr, "SFontTTF: BAD head table in %*.*s\n", SSARGS(name));
    return false;
  }
  TTF_HEAD* head_table = (TTF_HEAD*) tables[SS_TB_HEAD];
  longOffsets = ntohs (head_table->indexToLocFormat);

  if (longOffsets != 0 && longOffsets != 1)
  {
    fprintf (stderr, "SFontTTF: BAD TTF file [%*.*s] - indexToLocFormat.\n",
       SSARGS(name));
    return false;
  }

  if (tables[SS_TB_HHEA] == 0)
  {
    fprintf (stderr, "SFontTTF: BAD hhea table in %*.*s\n", SSARGS(name));
    return false;
  }
  if (tables[SS_TB_GLYF] == 0)
  {
    fprintf (stderr, "SFontTTF: BAD glyf table in %*.*s\n", SSARGS(name));
    return false;
  }
  if (tables[SS_TB_CMAP] == 0)
  {
    fprintf (stderr, "SFontTTF: BAD cmap table in %*.*s\n", SSARGS(name));
    return false;
  }
  if (tables[SS_TB_HTMX] == 0)
  {
    fprintf (stderr, "SFontTTF: BAD htmx table in %*.*s\n", SSARGS(name));
    return false;
  }
  if (tables[SS_TB_LOCA] == 0)
  {
  }
  TTF_POST_HEAD*  post_table = (TTF_POST_HEAD*) tables[SS_TB_POST];
  if (post_table == 0)
  {
    fprintf (stderr, "SFontTTF: missing post table in %*.*s. Using defaults\n",
         SSARGS(name));
    italicAngle = 0.0;
    underlineThickness = 100;
    underlinePosition = 0.0;
    isFixedPitch = 0.0;
  }
  else
  {
    italicAngle = (double) (ntohs(post_table->italicAngle.upper)) +
      (ntohs(post_table->italicAngle.lower) / 65536.0);
    underlineThickness = (double)ntohs(post_table->underlineThickness);
    underlinePosition = (double)ntohs(post_table->underlinePosition);
    isFixedPitch = (ntohl(post_table->isFixedPitch))? true : false;
  }
  TTF_HHEA* hhea_table = (TTF_HHEA*) tables[SS_TB_HHEA];
  scaleFactor = 1 / (double)((short)ntohs(head_table->unitsPerEm));

  lineGap = (double) ((short)htons (hhea_table->lineGap));
  charWidth = (double) ((short)(ntohs(head_table->xMax)+ ntohs(head_table->xMin)));

  charAscent = (double) ((short)htons (hhea_table->ascender));
  /* Be aware charDescent is negative! */
  charDescent = (double) ((short)htons (hhea_table->descender));
  return true;
}

/**
 * Draw a single unicode character on canvas using the pen.
 * before calling this, you should call a newpath and
 * after calling this you may want to call fill.
 * @param canvas is the canvas to draw to
 * @param m is the transformation matrix.
 * @param uch is the unicode character
 * @param len is the length of unicode array
 * @return the length in uch
 */
unsigned int
SFontTTF::draw (SCanvas* canvas, const SS_Matrix2D& matrix, 
  const SS_UCS4* _uch, unsigned int _len)
{
  if (!isOK()) return 0;
  SV_GlypIndex gi;
  unsigned int ret = findGlyphs (_uch, _len, &gi);
#if 0
if (_uch[0] == 0xfb2e)
{
  fprintf (stderr, "using SFontTTF::draw 0xfb2e ret=%d\n", ret);
}
#endif
  /**
   * Multiple glyphs draw on top of each other. 
   */
#if 0
arial dumps.
if (0x01FA == _uch[0])
{
  fprintf (stderr, "0x01FA glyph.composition = %u\n", ret);
}
#endif
  for (unsigned int i=0; i<ret; i++)
  {
    draw (canvas, matrix, gi[i]);
  }
  return ret;
}

/**
 * Draw a single unicode character on canvas using the pen.
 * before calling this, you should call a newpath and
 * after calling this you may want to call fill.
 * @param canvas is the canvas to draw to
 * @param m is the transformation matrix.
 * @param glyphno is the glyph to draw
 * @return the length in uch
 */
void
SFontTTF::draw (SCanvas* canvas, const SS_Matrix2D& matrix, SS_WORD16 glyphno)
{

  BYTE* glyf_start = (BYTE *) tables[SS_TB_GLYF];

  TTF_GLYF* gtable;
  int len =0;
  if (longOffsets)
  {
     ULONG* lloca = (ULONG *) tables[SS_TB_LOCA];
     gtable = (TTF_GLYF *) (glyf_start + ntohl (lloca[glyphno]));
     len = ntohl (lloca[glyphno+1]) - ntohl (lloca[glyphno]);
  }
  else
  {
     USHORT* sloca = (USHORT *) tables[SS_TB_LOCA];
     gtable = (TTF_GLYF *) (glyf_start + (ntohs (sloca[glyphno]) * 2));
     len = (ntohs (sloca[glyphno+1]) - ntohs (sloca[glyphno])) * 2;
  }
  TTF_GLYF* kludge = 0;
  if ((((unsigned long) gtable) & 1) != 0)
  {
    if (!broken)
    {
      fprintf  (stderr, "SFontTTF: fixing unaligned %*.*s.\n", SSARGS(name));
      broken = true;
    }
    kludge = new TTF_GLYF[len]; 
    CHECK_NEW (kludge);
    memcpy (kludge, gtable, len * sizeof (TTF_GLYF));
    gtable = kludge;
  }
  if (len != 0)
  {
    short ncontours = ntohs (gtable->numberOfContours);
    if (ncontours <= 0)
    {
      BYTE *ptr = ((BYTE *) gtable + sizeof(TTF_GLYF));
      SHORT *sptr = (SHORT *) ptr;
      /* Go through the contours and modyfy the transformation matrix */
      USHORT flagbyte;
      do
      {
        SS_Matrix2D m;
        flagbyte = ntohs(*sptr); sptr ++;
        SS_WORD16 glyphindex = ntohs(*sptr); sptr ++;

        if (flagbyte & ARG_1_AND_2_ARE_WORDS)
        {
           /* we need to make it short as it can be negative */
           m.t0 = (double) ((SHORT)ntohs(*sptr)); sptr++;
           m.t1 = (double) ((SHORT)ntohs(*sptr)); sptr++;
        }
        else
        {
           char* bptr = (char *)sptr;
           m.t0 = (signed char)bptr[0];
           m.t1 = (signed char)bptr[1];
           sptr ++;
        }
        if (flagbyte & WE_HAVE_A_SCALE)
        {
           m.x0 = m.y1 = f2dot14(*sptr);
           sptr ++;
        }
        else if (flagbyte & WE_HAVE_AN_X_AND_Y_SCALE)
        {
          m.x0 = f2dot14(*sptr); sptr ++;
          m.y1 = f2dot14(*sptr); sptr ++;
        }
        else if (flagbyte & WE_HAVE_A_TWO_BY_TWO)
        {
          m.x0 = f2dot14(*sptr); sptr ++;
          m.y0 = f2dot14(*sptr); sptr ++;
          m.x1 = f2dot14(*sptr); sptr ++;
          m.y1 = f2dot14(*sptr); sptr ++;
        }
        SS_Matrix2D mm = matrix * m;
        draw (canvas, mm, glyphindex, glyphno);
      } while (flagbyte & MORE_COMPONENTS);
    }
    else
    {
        draw (canvas, matrix, glyphno, glyphno);
    }
  }
  if (kludge) delete kludge;
  return;
}

/**
 * return the number of glyphs
 */
unsigned int
SFontTTF::length (const SS_UCS4* _uch, unsigned int _len)
{
  if (!isOK() || _uch==0) return 0;
  SS_UCS4 u4 = _uch[0];
  SV_GlypIndex gi;
  unsigned int len = 0;
  if (u4<=0xffff)
  {
    /* 2 bit per length  one of wich is indicator of set state */
    unsigned int r1 = (fastMap[u4/4] >> (2*(u4%4))) & 0x3;
    if (r1 > 0)
    {
      return r1-1;
    }
    len =  findGlyphs (_uch, _len, &gi);
    len = len % 3;
    fastMap[u4/4] |=  ((len+1) << (2*(u4%4)));
    return len;
  }
  len =  findGlyphs (_uch, _len, &gi);
  return len;
}

/**
 * Return the advacnce width of the glyphs
 * The value is the value that is multipied with matrix.
 * @param m is the transformation matrix.
 * @param uch is the unicode character
 * @param len is the length of unicode array
 * @param used will show how many characters were used in uch
 * @return the calibrated advace width
 */
double
SFontTTF::width (const SS_Matrix2D& m,
  const SS_UCS4* _uch, unsigned int _len, unsigned int* used)
{
  if (!isOK()) return 0;
  SV_GlypIndex gi;
  unsigned int ret = findGlyphs (_uch, _len, &gi);
  if (used) *used = ret;
  /**
   * Multiple glyphs draw on top of each other. 
   */
  double max = 0;
  /* We draw on top of each other... */
  for (unsigned int i=0; i<ret; i++)
  {
    double nwidth = width (m, gi[i]);
    if (nwidth > max) max = nwidth;
  }
  if (max < 0) return -max;
  return max;
}

/**
 * Find out the width, knowing the local glyph number
 * @param m is the transformation matrix
 * @param glyphno is the local glyph index in the glyph table.
 */
double
SFontTTF::width (const SS_Matrix2D& m, SS_WORD16 glyphno)
{
  if (!isOK()) return 0.0;
  TTF_HHEA* hhea_table = (TTF_HHEA*) tables[SS_TB_HHEA];
  LONGHORMETRIC* hmtx_entry = (LONGHORMETRIC*) tables[SS_TB_HTMX];

  int n_hmetrics = ntohs(hhea_table->numberOfHMetrics);
  FWORD* lsblist = (FWORD *) &hmtx_entry[n_hmetrics];
  
  short w;
  short lsb;

  if (glyphno >= n_hmetrics)
  {
    w = ntohs (hmtx_entry[n_hmetrics-1].advanceWidth);
    lsb = ntohs (lsblist[glyphno-n_hmetrics]);
  }
  else
  {
    w = ntohs(hmtx_entry[glyphno].advanceWidth);
    lsb = ntohs(hmtx_entry[glyphno].lsb);
  }
  return double (w) * m.x0;
}

/**
 * Return the calibrated ascent
 * @param m is the transformation matrix.
 */
double
SFontTTF::ascent (const SS_Matrix2D& m)
{
  if (!isOK()) return 0.0;
  double rvle =  charAscent * m.y1;
  if (rvle < 0)
  {
    rvle = -rvle;
  }
  /* FIXME: THIS IS A HACK. I DID NOT HAVE SPEC. */
  rvle = rvle - descent (m);
  if (rvle < 0)
  {
    fprintf (stderr, "STDERR FIXME! SFontTTF.cpp\n");
    return 1;
  }
  return rvle;
}

/**
 * Return the calibrated descent
 * @param m is the transformation matrix.
 */
double
SFontTTF::descent (const SS_Matrix2D& m)
{
  if (!isOK()) return 0.0;
  double rvle =  charDescent * m.y1;
  if (rvle < 0) return -rvle;
  return rvle;
}

/**
 * Return the calibrated avarage width
 * @param m is the transformation matrix.
 */
double
SFontTTF::width (const SS_Matrix2D& m)
{
  if (!isOK()) return 0.0;
  double rvle =  charWidth * m.x0;
  if (rvle < 0) return -rvle;
  return rvle;
}

/**
 * Return the calibrated gap
 * @param m is the transformation matrix.
 */
double
SFontTTF::gap (const SS_Matrix2D& m) 
{
  if (!isOK()) return 0.0;
  double rvle =  lineGap * m.y1;
  if (rvle < 0) return -rvle;
  return rvle;
}

/**
 * Return the scale factor. You multiply this with point size you want.
 * matrix diagonals for a 10 point font is scale, scale
 */
double
SFontTTF::scale ()
{
  if (!isOK()) return 0.0;
  return scaleFactor;
}

/**
 * This routine tries to find the glyph indeces of a unicode input stream
 * @param in is the input stream
 * @param len is the length if in
 * @param out is the output 
 * @return the nuber of characters processed in 'in'
 */
unsigned int
SFontTTF::findGlyphs (const SS_UCS4* in, unsigned int len, SV_GlypIndex* out)
{
  if (!isOK()) return 0;
  if (len == 0) return 0;
  TTF_CMAP* cmap_table = (TTF_CMAP*) tables[SS_TB_CMAP];
  int num_tables = ntohs(cmap_table->numberOfEncodingTables);
  SStringVector maps; 

  for (int i=0; i < num_tables; i++)
  {
    TTF_CMAP_ENTRY* table_entry = &(cmap_table->encodingTable[i]);
    int offset = ntohl(table_entry->offset);
    TTF_CMAP_FMT4* encoding4 = (TTF_CMAP_FMT4 *) ((BYTE *)cmap_table + offset);
    int format = ntohs(encoding4->format);
    //int platform = ntohs(table_entry->platformID);
    //int encoding_id = ntohs(table_entry->encodingID);

    /* I can understand only this format... */
    if (format != 4) continue;
    SUniMap umap = umaps[i];
    if (umaps.size() <= (unsigned int)i)
    {
     fprintf (stderr, "num_tables=%d i=%d mapsize=%u\n", num_tables, i, umaps.size());
    }
    if (!umap.isOK())
    {
      return 0;
    }
    /* get the encoder for this table. */
    SS_UCS2 ucs2 = umap.encode (in[0]);
    if (ucs2==0) continue;

    /* Finally we found it. Maybe */
    int seg_c2 = ntohs(encoding4->segCountX2);
    SHORT cmap_n_segs = seg_c2 >> 1;
    BYTE* ptr = (BYTE *)encoding4 + 14;
    USHORT* cmap_seg_end = (USHORT *) ptr;
    USHORT* cmap_seg_start = (USHORT *) (ptr + seg_c2 + 2);
    SHORT* cmap_idDelta = (SHORT *) (ptr + (seg_c2 * 2 )+ 2);
    SHORT* cmap_idRangeOffset = (SHORT *) (ptr + (seg_c2 * 3) + 2);

    /* No choice. Go through the segments */
    for (int j=0; j < cmap_n_segs-1; j++)
    {
      USHORT start = ntohs(cmap_seg_start[j]);
      USHORT end   = ntohs(cmap_seg_end[j]);
      SHORT delta = ntohs(cmap_idDelta[j]);
      USHORT ro    = ntohs(cmap_idRangeOffset[j]);

      if (ucs2> end || ucs2 < start) continue;

      /* K is the native index. it should be here  */
      for (int k = start; k <= end; k++)
      {
        USHORT n = (delta != 0) ? k + delta 
          : (USHORT) ntohs(*( (ro >> 1) + (k - start) +
            &(cmap_idRangeOffset [j])));
        if (ucs2 != k) continue;
        /* We really found it.. */
        if (n==0) return 0;
        out->append (n);
        /* Currently only one glyph is supported. later use  lengthIsOne */
#if 0
if (ucs2 == 0xfb2e)
{
  fprintf (stderr, "using SFontTTF::fingGlyphs 0xfb2e start=%u end=%d delta=%d ro=%u n=%u k=%d\n", (unsigned int) start, (unsigned int) end, (int) delta, (unsigned int) ro, (unsigned int) n, k);
}
#endif
        return 1;
      }
    }
  }
  //fprintf (stderr, "SFontTTF: no match. tried: %*.*s\n", SSARGS (maps.join(",")));
  return 0;
}

/**
 * Draw a single glyph.
 * @pama canvas is the canvas to draw to
 * @param pen is the pen to draw with
 * @param m is the transformation matrix.
 * @param glyphindex is the local glyph index
 * @return the advance width.
 */
void
SFontTTF::draw (SCanvas* canvas, const SS_Matrix2D& matrix, 
  SS_WORD16 glyphno, SS_WORD16 parent)
{
  if (!isOK()) return ;
  BYTE* gstart = (BYTE *) tables[SS_TB_GLYF];

  TTF_GLYF* gtable;
  int len =0;
  if (longOffsets)
  {
     ULONG* lloca = (ULONG *) tables[SS_TB_LOCA];
     gtable = (TTF_GLYF *) (gstart + ntohl (lloca[glyphno]));
     len = ntohl (lloca[glyphno+1]) - ntohl (lloca[glyphno]);
  }
  else
  {
     USHORT* sloca = (USHORT *) tables[SS_TB_LOCA];
     gtable = (TTF_GLYF *) (gstart + (ntohs (sloca[glyphno]) << 1));
     len = (ntohs (sloca[glyphno+1]) - ntohs (sloca[glyphno])) << 1;
  }
  if (len <= 0)
  {
    if (!broken)
    {
      fprintf (stderr, "SFontTTF: non-existent glyph %u/%u in %*.*s\n",
          (unsigned int) parent, 
          (unsigned int) glyphno,
          SSARGS (name)) ;
      broken = true;
    }
    return;
  }
  TTF_GLYF* kludge = 0;
  if ((((unsigned long) gtable) & 1) != 0)
  {
    if (!broken)
    {
      fprintf  (stderr, "SFontTTF: fixing unaligned %*.*s.\n", SSARGS(name));
      broken = true;
    }
    kludge = new TTF_GLYF[len]; 
    CHECK_NEW (kludge);
    memcpy (kludge, gtable, len * sizeof (TTF_GLYF));
    gtable = kludge;
  }

  int ncontours = (int) ((short)ntohs (gtable->numberOfContours));
  if (ncontours <= 0)
  {
    fprintf (stderr, "SFontTTF: non-existent glyph %u/%u in %*.*s\n",
        (unsigned int) parent, 
        (unsigned int) glyphno,
        SSARGS (name)) ;
    if (kludge) delete kludge;
    return;
  }

  USHORT* contour_end_pt = (USHORT *) ((char *)gtable + sizeof(TTF_GLYF));

  int last_point = (int) ntohs (contour_end_pt[ncontours-1]);

  int n_inst = (int) ntohs (contour_end_pt[ncontours]);

  BYTE* ptr = ((BYTE *)contour_end_pt) + (ncontours << 1) + n_inst + 2;

  int j = 0; int k = 0;

  SBinVector<BYTE> flags;
  while (k <= last_point)
  {
    flags.append (ptr[j]);
    if (ptr[j] & REPEAT)
    {
       for (int k1=0; k1 < ptr[j+1]; k1++)
       {
            k++;
            flags.append (ptr[j]);
       }
       j++;
    }
    j++; k++;
  }
  
  SBinVector<SHORT> xrel;
  SBinVector<SHORT> xcoord;

  for (k=0; k <= last_point; k++)
  {
    /* Process xrel */
    if (flags[k] & XSHORT)
    {
      if (flags[k] & XSAME)
      {
        xrel.append (ptr[j]);
      }
      else
      {
        xrel.append (-ptr[j]);
      }
      j++;
    }
    else if (flags[k] & XSAME)
    {
      xrel.append (0);
    }
    else
    {
      xrel.append (ptr[j] * 256 + ptr[j+1]);
      j += 2;
    }
    /* Process x coordinate */
    if (k==0)
    {
      xcoord.append (xrel[k]);
    }
    else
    {
      xcoord.append (xrel[k] + xcoord[k-1]);
    }
  }

  SBinVector<SHORT> yrel;
  SBinVector<SHORT> ycoord;

  /* one more run fore yrel and ycoord */
  for (k=0; k <= last_point; k++)
  {
    if (flags[k] & YSHORT)
    {
      if (flags[k] & YSAME)
      {
        yrel.append (ptr[j]);
      }
      else
      {
         yrel.append (- ptr[j]);
      }
      j++;
    }
    else if (flags[k] & YSAME)
    {
      yrel.append (0);
    }
    else
    {
      yrel.append (ptr[j] * 256 + ptr[j+1]);
      j += 2;
    }
    if (k==0)
    {
       ycoord.append (yrel[k]);
    }
    else
    {
       ycoord.append (yrel[k] + ycoord[k-1]);
    }
  }

  bool first = true;

  j = 0;
  int i = 0;
  int contour_start = 0;
  while (i <= ntohs(contour_end_pt[ncontours-1]))
  {
    int contour_end = ntohs(contour_end_pt[j]);

    if (first)
    {
      moveto (canvas, matrix, xcoord[i], ycoord[i]);
      contour_start = i;
      first = false;
    }
    else if (flags[i] & ONOROFF)
    {
      lineto (canvas, matrix, xcoord[i], ycoord[i]);
    }
    else /* curvy */
    {
      int cs = i-1;
      int ce = 0;
      bool finished = false;
      int nguide = 0;
      while (!finished)
      {
        if (i == contour_end+1)
        {
          ce = contour_start;
          finished = true;
        }
        else if (flags[i] & ONOROFF)
        {
          ce = i;
          finished = true;
        }
        else
        {
           i++;
           nguide++;
        }
      }

      switch (nguide)
      {
      case 0:
        lineto (canvas, matrix, xcoord[ce], ycoord[ce]);
        break;

      case 1:
        cureveto (canvas, matrix, 
           (xcoord[cs]+2*xcoord[cs+1])/3, (ycoord[cs]+2*ycoord[cs+1])/3,
           (2*xcoord[cs+1]+xcoord[ce])/3, (2*ycoord[cs+1]+ycoord[ce])/3,
            xcoord[ce], ycoord[ce]);
        break;

      case 2: 
        cureveto (canvas, matrix, 
            (-xcoord[cs]+4*xcoord[cs+1])/3, (-ycoord[cs]+4*ycoord[cs+1])/3,
            (4*xcoord[cs+2]-xcoord[ce])/3, (4*ycoord[cs+2]-ycoord[ce])/3,
            xcoord[ce], ycoord[ce]);
        break;

      case 3: 
        cureveto (canvas, matrix, 
            (xcoord[cs]+2*xcoord[cs+1])/3, (ycoord[cs]+2*ycoord[cs+1])/3,
            (5*xcoord[cs+1]+xcoord[cs+2])/6, (5*ycoord[cs+1]+ycoord[cs+2])/6,
            (xcoord[cs+1]+xcoord[cs+2])/2, (ycoord[cs+1]+ycoord[cs+2])/2);
        cureveto (canvas, matrix,
            (xcoord[cs+1]+5*xcoord[cs+2])/6, (ycoord[cs+1]+5*ycoord[cs+2])/6,
            (5*xcoord[cs+2]+xcoord[cs+3])/6, (5*ycoord[cs+2]+ycoord[cs+3])/6,
            (xcoord[cs+3]+xcoord[cs+2])/2, (ycoord[cs+3]+ycoord[cs+2])/2);
        cureveto (canvas, matrix,
            (xcoord[cs+2]+5*xcoord[cs+3])/6, (ycoord[cs+2]+5*ycoord[cs+3])/6,
            (2*xcoord[cs+3]+xcoord[ce])/3, (2*ycoord[cs+3]+ycoord[ce])/3,
            xcoord[ce], ycoord[ce]);
        break;

      default:
        int k1 = cs + nguide;
        cureveto (canvas, matrix, 
            (xcoord[cs]+2*xcoord[cs+1])/3, (ycoord[cs]+2*ycoord[cs+1])/3,
            (5*xcoord[cs+1]+xcoord[cs+2])/6, (5*ycoord[cs+1]+ycoord[cs+2])/6,
            (xcoord[cs+1]+xcoord[cs+2])/2, (ycoord[cs+1]+ycoord[cs+2])/2);
        for (k = cs+2; k <= k1-1; k++)
        {
          cureveto (canvas, matrix, 
            (xcoord[k-1]+5*xcoord[k])/6, (ycoord[k-1]+5*ycoord[k])/6,
            (5*xcoord[k]+xcoord[k+1])/6, (5*ycoord[k]+ycoord[k+1])/6,
            (xcoord[k]+xcoord[k+1])/2, (ycoord[k]+ycoord[k+1])/2);
        }
        cureveto (canvas, matrix, 
            (xcoord[k1-1]+5*xcoord[k1])/6, (ycoord[k1-1]+5*ycoord[k1])/6,
            (2*xcoord[k1]+xcoord[ce])/3, (2*ycoord[k1]+ycoord[ce])/3,
            xcoord[ce], ycoord[ce]);
        break;
      } /* end switch (nguide) */
    }
    if (i >= contour_end)
    {
      first = true;
      i = contour_end + 1;
      canvas->closepath ();
      j++;
    }
    else
    {
      i++;
    }
  }
  if (kludge!=0) delete kludge;
  return;
}

/**
 * The moveto
 */
static void
moveto (SCanvas* canvas, const SS_Matrix2D &m,
  SHORT _x, SHORT _y)
{
  double x = m.x0 * double (_x) + m.y0 * double (_y) + m.t0;
  double y = m.x1 * double (_x) + m.y1 * double (_y) + m.t1;
  canvas->moveto (x, y);
//fprintf (stderr, "moveto: %g,%g\n", x, y);
}
/**
 * The lineto
 */
static void
lineto (SCanvas* canvas, const SS_Matrix2D &m,
  SHORT _x, SHORT _y)
{
  double x = m.x0 * double (_x) + m.y0 * double (_y) + m.t0;
  double y = m.x1 * double (_x) + m.y1 * double (_y) + m.t1;
  canvas->lineto (x, y);
//fprintf (stderr, "lineto: %g,%g\n", x, y);
}

/**
 * The curveto
 */
static void
cureveto (SCanvas* canvas, const SS_Matrix2D &m,
  SHORT _x0, SHORT _y0,
  SHORT _x1, SHORT _y1,
  SHORT _x2, SHORT _y2)
{
  double x0 = m.x0 * double (_x0) + m.y0 * double (_y0) + m.t0;
  double y0 = m.x1 * double (_x0) + m.y1 * double (_y0) + m.t1;

  double x1 = m.x0 * double (_x1) + m.y0 * double (_y1) + m.t0;
  double y1 = m.x1 * double (_x1) + m.y1 * double (_y1) + m.t1;

  double x2 = m.x0 * double (_x2) + m.y0 * double (_y2) + m.t0;
  double y2 = m.x1 * double (_x2) + m.y1 * double (_y2) + m.t1;
  canvas->curveto (x0, y0, x1, y1, x2, y2);
//fprintf (stderr, "curveto: %g,%g %g,%g %g,%g\n", x0, y0, x1, y1, x2, y2);
}

/**
 * Create double numbers between -2 .. 1.99994 
 * from a packed short.
 */
static double
f2dot14 (short x)
{
  short y = ntohs(x);
  return (y >> 14) + ((y & 0x3fff) / 16384.0);
}
