/** 
 *  Yudit Unicode Editor Source File
 *
 *  Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#ifndef SFontTTF_h
#define SFontTTF_h

#include "swindow/SCanvas.h"
#include "swindow/SPen.h"

#include "stoolkit/SIO.h"
#include "stoolkit/STypes.h"
#include "stoolkit/SMatrix.h"
#include "stoolkit/SBinHashtable.h"
#include "stoolkit/SBinVector.h"
#include "stoolkit/SString.h"
#include "stoolkit/SProperties.h"
#include "stoolkit/SUniMap.h"


/**
 * This file will not be exposed so we can put some local junk here.
 */

/**
 * @author: Gaspar Sinai <gsinai@yudit.org>
 * @version: 2000-04-23
 * This is the abstract widget toolkit font package
 */

typedef SBinVector<SS_WORD16> SV_GlypIndex;

class SFontTTF
{
public:
  SFontTTF (const SFile& file); 
  virtual ~SFontTTF (); 

  unsigned int draw (SCanvas* canvas, 
    const SS_Matrix2D& matrix, const SS_UCS4* uch, unsigned int len);

  double width (const SS_Matrix2D& matrix, const SS_UCS4* uch,
      unsigned int len, unsigned int* used);
  double width (const SS_Matrix2D& matrix);
  double ascent (const SS_Matrix2D& matrix);
  double descent (const SS_Matrix2D& matrix);
  double gap (const SS_Matrix2D& matrix);

  /* mulltiply this with size you want to get the matrix */
  double scale ();

  void draw (SCanvas* canvas, const SS_Matrix2D& matrix, SS_WORD16 glyphno);

  void draw (SCanvas* canvas, const SS_Matrix2D& matrix, 
    SS_WORD16 glyphindex, SS_WORD16 glyphno);

  bool isOK();
  unsigned int length (const SS_UCS4* in, unsigned int len);
private:
  unsigned char* fastMap;
  double width (const SS_Matrix2D& m, SS_WORD16 glyphno);
  unsigned int findGlyphs (const SS_UCS4* in, 
       unsigned int len, SV_GlypIndex* out);
  SFileImage image;
  SString name;
  SFile file;
  bool ok;
  bool broken;

  bool init ();
  bool processName (); 
  bool checkTables ();
  void getName (long id, const char* str, int len);

  SProperties          names;
  SBinHashtable<void*> tables;
  SVector<SUniMap>     umaps;
  bool                 lengthIsOne;
  
  //double  fontSize;
  double  scaleFactor;

  // line height is charAscent - charDescent + lineGap
  double  lineGap;
  double  charWidth;
  double  charAscent;
  double  charDescent;

  //double  scaleFactor;

  double  italicAngle;
  double  underlineThickness;
  double  underlinePosition;
  bool    isFixedPitch;
  int     longOffsets;
};

#endif /* SFontTTF_h */
