/** 
 *  Yudit Unicode Editor Source File
 *
 *  Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#include "stoolkit/SExcept.h"
#include "stoolkit/SEvent.h"
#include "stoolkit/SString.h"
#include "stoolkit/SStringVector.h"
#include "stoolkit/SBinHashtable.h"
#include "swindow/SAwt.h"
#include "swindow/swin32/SWin32.h"

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#undef WIN32_LEAN_AND_MEAN 

#include <stdio.h>

#ifdef USE_WINAPI
int main(int argc, char* argv[]);
#endif


/**
 * @author: Gaspar Sinai <gsinai@yudit.org>
 * @version: 2000-04-23
 * This is the abstract widget toolkit
 */

/* There is only one context */
HINSTANCE instance;
HINSTANCE pinstance;
LPSTR cmdLine;
int cmdShow;
HANDLE accel;

static bool winOK = false;

typedef SBinHashtable<SW32Window*> SWindowHashtable;
typedef SBinHashtable<SWindowListener*> SListenerHashtable;
static SWindowHashtable windowHashtable;
static SListenerHashtable listenerHashtable;

char* windowName = "Main";

class SWHandler : public SJob, SEventTarget
{
public:
  SWHandler(void);
  ~SWHandler();
  virtual int run();
  virtual bool timeout(SEventSource* s);
private:
  STimer timer;
};


SWHandler::SWHandler(void) : SJob(1), timer (100)
{
  SEventHandler::addTimer(&timer, this);
  SEventHandler::addJob(this, this);
}

SWHandler::~SWHandler()
{
}

int
SWHandler::run()
{
  if (!winOK)
  {
    return -1;
  }
  //fprintf (stderr, "RUN\n");
  MSG msg;
  if( GetMessage(&msg ,0 , 0, 0)<=0)
  {
    return 0;
  }
  //if (!TranslateAccelarator(hwnd, accel,&msg)
  TranslateMessage(&msg);
  DispatchMessage(&msg);
  return 1;
}

bool
SWHandler::timeout(SEventSource* s)
{
  return winOK;
}

LRESULT CALLBACK _eventHandler(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

/**
 * This is wher ewe start off
 */
int APIENTRY
WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
  winOK = true;
  instance = hInstance;
  pinstance = hPrevInstance;
  cmdLine = lpszCmdLine;
  cmdShow = nCmdShow;

  WNDCLASS wcl;
  wcl.hInstance = instance;
  wcl.lpszClassName=windowName;
  wcl.lpfnWndProc = _eventHandler;
  wcl.style = 0;
  wcl.hIcon = LoadIcon(0, IDI_APPLICATION);
  wcl.hCursor=LoadCursor(0, IDC_ARROW);
  wcl.lpszMenuName = 0;
  wcl.cbClsExtra = 0;
  wcl.cbWndExtra = 0;
  //wcl.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
  //wcl.hbrBackground = GetSysColorBrush(WHITE_BRUSH);
  //wcl.hbrBackground = GetSysColorBrush(WHITE_BRUSH);
  wcl.hbrBackground = GetSysColorBrush(COLOR_WINDOW);
  //wcl.hbrBackground = GetSysColorBrush(COLOR_BACKGROUND);
  if (!RegisterClass(&wcl)) return 0;

  char* cl = GetCommandLine();
  SStringVector l(cl, "\t ");
  int argc = l.size();

  char **argv = new char*[argc+1];
  for (unsigned int i=0;i<argc; i++)
  {
    argv[i] = l[i].cString();
  }
  argv[i] = 0;
  accel = LoadAccelerators(instance, "Main");
  int ret;
  {
    SWHandler h;
    ret=main (argc, argv);
  }
  for (unsigned int j=0;j<argc; j++)
  {
    delete argv[j];
  }
  delete argv;
  return ret;
}


SW32Impl::SW32Impl()
{
}

SW32Impl::~SW32Impl()
{
}

bool
SW32Impl::isOK()
{
  return winOK;
}

SWindow*
SW32Impl::getWindow (SWindowListener* l, const SString& name)
{
  if (!isOK()) return 0;
  char* nm=name.cString();
  HWND w = CreateWindow(
     windowName, nm,
     WS_OVERLAPPEDWINDOW,
     100, 100, 600, 400,
/*
     CW_USEDEFAULT, CW_USEDEFAULT, 
     CW_USEDEFAULT, CW_USEDEFAULT, 
*/
     HWND_DESKTOP,
     0,
     instance,
     0);
  delete nm;
  SW32Window* sw = new SW32Window (name, this, (long) w);
  CHECK_NEW (sw);
  listenerHashtable.put ((long) w, l);
  return sw;
}

SW32Window::SW32Window(const SString& n, SW32Impl* i, long _id) : name(n)
{
  impl = i;
  id = _id;
  windowHashtable.put (id, this);
}

SW32Window::~SW32Window()
{
  windowHashtable.remove (id);
  listenerHashtable.remove (id);
}

void
SW32Window::show ()
{
  ShowWindow((HWND)id, cmdShow);
  UpdateWindow((HWND)id);
}
/**
 * This reqests a redraw, efficiently after all events got processed.
 * @param clear is true if the window needs to be cleared before calling redraw.
 * @param x is the x origin of the event
 * @param y is the y origin of the event
 * @param width is the width of the event
 * @param height is the height of the event
 */
void
SW32Window::redraw (bool clear, int x, int y, unsigned int width, unsigned int height)
{
}

/**
 * Reparent the window. 
 * TODO: move it to x y
 * @param p is the parent window
 */
void
SW32Window::setParent (SWindow* p, int x, int y)
{
  SetParent ((HWND)id, (HWND)((SW32Window*)p)->id);
}

LRESULT CALLBACK
_eventHandler (HWND hwnd, UINT message, WPARAM wParam,LPARAM lParam)
{
  switch(message)
  {
  case WM_COMMAND:
#if 0
  switch(LOWORD(wParam))
  {
    //case mi_dialog1:
    //DialogBox(hInst,"MYDB", hwnd, DialogFunc);
    //break;
  
    //case mi_dialog2:
    //MessageBox(hwnd,"Dialog not implemented","Error",MB_OK);
    break;
  
  case mi_help:
    //MessageBox(hwnd,"The purpose of this program \n", MB_OK);
    break;
  }
#endif
  case WM_DESTROY:
    PostQuitMessage (0);
    SEventHandler::exit();
    break;
  default:
     return DefWindowProc(hwnd,message,wParam,lParam);
  }
  return DefWindowProc(hwnd,message,wParam,lParam);
}

#if 0
BOOL CALLBACK
_dialogFunc (HWND box,UINT message, WPARAM wParam,LPARAM lParam)
{
  switch(message)
  {
  case WM_COMMAND:
    switch (LOWORD(wParam))
    {
    case IDCANCEL:
     EndDialog(box,0);
     break;
    case db_red:
     MessageBox(box, "You Picked Red", "RED", MB_OK);
     break;
  case db_green:
    MessageBox(box,"You Picked Green","GREEN",MB_OK);
    break;
  }
  default:
     return FALSE;
  }
  return TRUE:
}
#endif
